import java.io.*;

import no.rwr.x.*;
import no.rwr.xutil.*;
//import no.rwr.render.*;
import no.rwr.image.*;
import java.awt.image.*;
import java.awt.color.*;
import java.awt.Point;
import java.awt.Transparency;

public class Viewer {
    public static void main(String[] args) throws IOException {

	// Read image
	TgaParser tp = new TgaParser(new FileInputStream(args[0]));
	System.out.println("TGA parsing done"); // DEBUG

	// Retrieve information about image (data and format)
	int w = tp.width;
	int h = tp.height;
	DataBufferByte tgaData = tp.data;             // Data
	ColorModel tgaCM = tp.colorModel;             // Color model
	ComponentSampleModel tgaCSM = tp.sampleModel; // Sample model
	WritableRaster tgaRaster = tp.wRaster;

	Raster raster = tgaRaster;

	// Decide where to display it
	Display display = new Display();
	Screen screen = display.getDefaultScreen();
	Window root = screen.getRootWindow();

	// FIXME: Make a point of finding the best visual
	Visual visual = screen.getRootVisual();

	long startMillis = System.currentTimeMillis();
	long startFilter = 0;
	long endFilter = 0;
	RasterOp renderer = null;

	// Create pixmap with image
	Pixmap pixmap;
	Image image;
	try {
	    
	    /* Create a factory that can create ximages based directly
	       on the data from the TGA image. If this works, the data
	       read will be sent directly to the display, without any
	       conversion. On my display, the 24bit mode of the
	       xserver matches the data format of the TGA image
	       exactly. In this case the data is used unmodified, and
	       only an ximage stucture describing the exact format is
	       created. */
	    XImageFactory factory = new XImageFactory(visual, tgaCM, tgaCSM);
	    image = new Image(factory, raster);
	    
	} catch (IllegalArgumentException iae) {
	    /* The data from the TGA image was not directly compatible
               with the formats supported by the dislay. Create a
               renderer that will convert the image data from the TGA
               format to a format supported by the display. */
	    
	    System.out.println("direct factory failed, trying renderer");

	    image = new Image(visual,
			      raster.getWidth(),
			      raster.getHeight());

	    XImageFactory factory = image.getXImageFactory();

	    /* Ask the Renderers utility class to find a renderer that
               can convert from the data format of the TGA image to
               the data format supported by the display. */
	    renderer =
		Renderers.getRenderer(tgaCM, tgaCSM,
				      factory.getColorModel(),
				      factory.getSampleModel());

	    WritableRaster imageRaster = image.getWritableRaster();

	    startFilter = System.currentTimeMillis();
	    renderer.filter(raster, imageRaster);
	    endFilter = System.currentTimeMillis();

	}
	
	pixmap = image.getPixmap();

	long endMillis = System.currentTimeMillis();

	long filterMillis = endFilter - startFilter;
	if (filterMillis != 0) {
	    System.out.println("Filtering took " +
			       filterMillis + " milliseconds.");
	    double pps = w*h; pps = pps/filterMillis;
	    System.out.println(renderer + " gave a throughput of " +
			       pps + " pixels per millisecond.");
	}

	System.out.println("Total time spent creating pixmap from raster " +
			   "was " + (endMillis - startMillis) +
			   " milliseconds.");


	// Create window and display image
	WindowAttributes frameAttr = new WindowAttributes();
	frameAttr.setBackground(pixmap);
	frameAttr.setVisual(visual);

	/* Do not allow window to be larger than the pixmap, since this
	   will show an ugly tiling effect. */
	WMSizeHints sizeHints = new WMSizeHints();
	sizeHints.setMaxSize(w, h);

	Frame frame = new Frame();
	frame.setTitle("Image viewer hack");
	frame.setSize(w, h);
	frame.setNormalSizeHints(sizeHints);
	frame.setParent(root);
	frame.setAttributes(frameAttr);
	frame.setVisible(true);


	// Run event loop
	Dispatcher dispatcher = new Dispatcher(display);
	dispatcher.run();
    }
}
