package no.rwr.image;

import java.awt.image.*;
import no.rwr.data.Buffers;

public class ComponentConvertOp implements RasterOp {

    protected ColorModel srcCM;
    protected ColorModel destCM;
    protected int numComponents;

    public ComponentConvertOp(ColorModel srcCM, ColorModel destCM) {
	this.srcCM  =  srcCM;
	this.destCM = destCM;
	this.numComponents = srcCM.getNumComponents();
    }
    

    public WritableRaster filter(Raster src, WritableRaster dest) {
	if (dest == null) dest = createCompatibleDestRaster(src);

	int w = src.getWidth();
	int h = src.getHeight();
	int size = w*h;

	int[] pixels = src.getPixels(src.getMinX(), src.getMinY(),
				     w, h, (int[]) null);

	float[] normalized = null;
	
	int offset = 0;
	for (int i=0; i<size; i++) {
	    normalized = srcCM.getNormalizedComponents(pixels,
						       offset,
						       normalized, 0);
	    destCM.getUnnormalizedComponents(normalized, 0,
					     pixels,
					     offset);
	    offset += numComponents;
	}
	
	dest.setPixels(dest.getMinX(), dest.getMinY(), w, h, pixels);
	return dest;	
    }

    public WritableRaster createCompatibleDestRaster(Raster src) {

	/* FIXME: Maybe we should explicitly create a raster with a
	   tightly pixel packed sample model, rather than assuming
	   that the createCompatibleWritableRaster() method in Raster
	   will create one. */

	return src.createCompatibleWritableRaster();
    }
}
