package no.rwr.image;

import java.awt.image.*;
import no.rwr.util.ArrayUtil;

public class Renderers {

    static final int[] MASKS_A5B6C5 = { 0xf800, 0x7e0, 0x1f };
    static final int[] COMPONENTS_ABC = { 0, 1, 2 };
    static final int[] COMPONENTS_CBA = { 2, 1, 0 };
    
    public static RasterOp getRenderer(ColorModel srcCM,
				       SampleModel srcSM,
				       ColorModel destCM,
				       SampleModel destSM) {
	
	if (srcCM.equals(destCM)) {
	    //System.out.println("no color conversion needed"); // DEBUG
	    return getRenderer(srcSM, destSM);
	}	

	if (srcCM.getColorSpace().equals(destCM.getColorSpace())) {
	    //System.out.println("same color space"); // DEBUG
	    //System.out.println("component conversion needed"); // DEBUG

	    /* FIXME: implement more efficient color converters that
	       handle the common cases:
	       
	       1. sRGB ComponentColorModel ---> DirectColor    (RGB packing)
	       
	       2. DirectColor ---> DirectColor                 (RGB repacking)
	       
	       3. BGR ComponentColorModel <---> sRGB ComponentColorModel

	    */
	    int numComponents = destCM.getNumComponents();
	    if ((srcCM instanceof ComponentColorModel) &&
		(destCM instanceof PackedColorModel) && 
		(numComponents == 3) &&
		(srcCM.getNumComponents() == 3)) {

		ComponentColorModel srcCCM = (ComponentColorModel) srcCM;
		PackedColorModel destPCM = (PackedColorModel) destCM;

		ComponentSampleModel srcCSM = (ComponentSampleModel) srcSM;

		if ((srcSM.getDataType() == DataBuffer.TYPE_BYTE) &&
		    (destSM.getDataType() == DataBuffer.TYPE_USHORT)) {
		    

		    /* FIXME: Use Class.forName(...) to obtain class based
		       on name. */

		    int[] masks = destPCM.getMasks();
		    if (ArrayUtil.equals(masks, MASKS_A5B6C5)) {
			System.out.println("MASKS_A5B6C5");


			int[] bandOffsets = srcCSM.getBandOffsets();
			
			if (ArrayUtil.equals(bandOffsets, COMPONENTS_ABC)) {
			    System.out.println("COMPONENTS_ABC");
			    return new RenderCSMByteABCToSPPSMUShort_5A6B5C();
			}

			if (ArrayUtil.equals(bandOffsets, COMPONENTS_CBA)) {
			    System.out.println("COMPONENTS_CBA");
			    return new RenderCSMByteCBAToSPPSMUShort_5A6B5C();
			}
		    }

		    
		    
		    //System.out.println("using ShortFieldPacker"); // DEBUG
		    
		    return new ShortFieldPacker(srcCSM, destPCM);
		}
		
		



	    }

	    //return new ComponentConvertOp(srcCM, destCM);
	}

	//System.out.println("color conversion needed"); // DEBUG
	return new ColorConvertOp(srcCM, destCM);
    }

    public static RasterOp getRenderer(SampleModel srcSM,
				       SampleModel destSM) {
	int numBands = srcSM.getNumBands();
	if (destSM.getNumBands() != numBands) {
	    throw new IllegalArgumentException("The number of the bands" +
					       " of the sample models " +
					       "do not match (" +
					       numBands + "!=" +
					       destSM.getNumBands() + ")");
	}

	testForMatchingPackedSamples:
	{
	    if (!((srcSM  instanceof ComponentSampleModel) &&
		  (destSM instanceof ComponentSampleModel)))
		break testForMatchingPackedSamples;

	    //System.out.println("both are ComponentSampleModel"); // DEBUG

	    ComponentSampleModel  srcCSM = (ComponentSampleModel)  srcSM;
	    ComponentSampleModel destCSM = (ComponentSampleModel) destSM;
	
	    int pixelStride = srcCSM.getPixelStride();
	    if (pixelStride != destCSM.getPixelStride())
		break testForMatchingPackedSamples;

	    //System.out.println("pixel stride match"); // DEBUG

	    if (pixelStride > numBands)
		break testForMatchingPackedSamples;

	    //System.out.println("pixel stride indicate interleaving"); // DEBUG

	    int[]  srcOffsets =  srcCSM.getBandOffsets();
	    int[] destOffsets = destCSM.getBandOffsets();
	    boolean[] coverage = new boolean[numBands];
	    for (int b=0; b<numBands; b++) {
		int offset = srcOffsets[b];
		if (offset != destOffsets[b])
		    break testForMatchingPackedSamples;
		if (offset >= numBands)
		    break testForMatchingPackedSamples;
		coverage[offset] = true;
	    }
	    for (int b=0; b<numBands; b++)
		if (!coverage[b]) break testForMatchingPackedSamples;
	    
	    //System.out.println("the sample models interleave and" + // DEBUG
	    //		       " pack the samples the same way");    // DEBUG
	    return new ComponentDataBlitOp();
	}
	
	//System.out.println("pixel copying needed");
	return new PixelCopyOp();
    }

}
